@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -15,7 +15,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <http://www.gnu.org/licenses/>.
+# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
 
 """Initialization of qutebrowser and application-wide things.
 
@@ -39,10 +39,11 @@ blocks and spins the Qt mainloop.
 
 import os
 import sys
+import functools
 import tempfile
 import datetime
 import argparse
-import typing
+from typing import Iterable, Optional, cast
 
 from PyQt5.QtWidgets import QApplication, QWidget
 from PyQt5.QtGui import QDesktopServices, QPixmap, QIcon
@@ -51,7 +52,8 @@ from PyQt5.QtCore import pyqtSlot, QUrl, QObject, QEvent, pyqtSignal, Qt
 import qutebrowser
 import qutebrowser.resources
 from qutebrowser.commands import runners
-from qutebrowser.config import config, websettings, configfiles, configinit
+from qutebrowser.config import (config, websettings, configfiles, configinit,
+                                qtargs)
 from qutebrowser.browser import (urlmarks, history, browsertab,
                                  qtnetworkdownloads, downloads, greasemonkey)
 from qutebrowser.browser.network import proxy
@@ -59,12 +61,12 @@ from qutebrowser.browser.webkit import cookies, cache
 from qutebrowser.browser.webkit.network import networkmanager
 from qutebrowser.extensions import loader
 from qutebrowser.keyinput import macros, eventfilter
-from qutebrowser.mainwindow import mainwindow, prompt
+from qutebrowser.mainwindow import mainwindow, prompt, windowundo
 from qutebrowser.misc import (ipc, savemanager, sessions, crashsignal,
                               earlyinit, sql, cmdhistory, backendproblem,
                               objects, quitter)
 from qutebrowser.utils import (log, version, message, utils, urlutils, objreg,
-                               usertypes, standarddir, error, qtutils)
+                               usertypes, standarddir, error, qtutils, debug)
 # pylint: disable=unused-import
 # We import those to run the cmdutils.register decorators.
 from qutebrowser.mainwindow.statusbar import command
@@ -72,7 +74,7 @@ from qutebrowser.misc import utilcmds
 # pylint: enable=unused-import
 
 
-q_app = typing.cast(QApplication, None)
+q_app = cast(QApplication, None)
 
 
 def run(args):
@@ -80,6 +82,8 @@ def run(args):
     if args.temp_basedir:
         args.basedir = tempfile.mkdtemp(prefix='qutebrowser-basedir-')
 
+    log.init.debug("Main process PID: {}".format(os.getpid()))
+
     log.init.debug("Initializing directories...")
     standarddir.init(args)
     utils.preload_resources()
@@ -88,19 +92,20 @@ def run(args):
     configinit.early_init(args)
 
     log.init.debug("Initializing application...")
-    global q_app
-    q_app = Application(args)
-    q_app.setOrganizationName("qutebrowser")
-    q_app.setApplicationName("qutebrowser")
-    q_app.setDesktopFileName("org.qutebrowser.qutebrowser")
-    q_app.setApplicationVersion(qutebrowser.__version__)
+    app = Application(args)
+    objects.qapp = app
+    app.setOrganizationName("qutebrowser")
+    app.setApplicationName("qutebrowser")
+    # Default DesktopFileName is org.qutebrowser.qutebrowser, set in `get_argparser()`
+    app.setDesktopFileName(args.desktop_file_name)
+    app.setApplicationVersion(qutebrowser.__version__)
 
     if args.version:
         print(version.version_info())
         sys.exit(usertypes.Exit.ok)
 
     quitter.init(args)
-    crashsignal.init(q_app=q_app, args=args, quitter=quitter.instance)
+    crashsignal.init(q_app=app, args=args, quitter=quitter.instance)
 
     try:
         server = ipc.send_or_listen(args)
@@ -131,7 +136,7 @@ def qt_mainloop():
     WARNING: misc/crashdialog.py checks the stacktrace for this function
     name, so if this is changed, it should be changed there as well!
     """
-    return q_app.exec_()
+    return objects.qapp.exec()
 
 
 def init(*, args: argparse.Namespace) -> None:
@@ -140,10 +145,11 @@ def init(*, args: argparse.Namespace) -> None:
 
     crashsignal.crash_handler.init_faulthandler()
 
-    q_app.setQuitOnLastWindowClosed(False)
+    objects.qapp.setQuitOnLastWindowClosed(False)
     quitter.instance.shutting_down.connect(QApplication.closeAllWindows)
 
     _init_icon()
+    _init_pulseaudio()
 
     loader.init()
     loader.load_components()
@@ -159,7 +165,7 @@ def init(*, args: argparse.Namespace) -> None:
     eventfilter.init()
 
     log.init.debug("Connecting signals...")
-    q_app.focusChanged.connect(on_focus_changed)
+    objects.qapp.focusChanged.connect(on_focus_changed)
 
     _process_args(args)
 
@@ -185,7 +191,23 @@ def _init_icon():
     if icon.isNull():
         log.init.warning("Failed to load icon")
     else:
-        q_app.setWindowIcon(icon)
+        objects.qapp.setWindowIcon(icon)
+
+
+def _init_pulseaudio():
+    """Set properties for PulseAudio.
+
+    WORKAROUND for https://bugreports.qt.io/browse/QTBUG-85363
+
+    Affected Qt versions:
+    - Older than 5.11 (which is unsupported)
+    - 5.14.0 to 5.15.0 (inclusive)
+
+    However, we set this on all versions so that qutebrowser's icon gets picked
+    up as well.
+    """
+    for prop in ['application.name', 'application.icon_name']:
+        os.environ['PULSE_PROP_OVERRIDE_' + prop] = 'qutebrowser'
 
 
 def _process_args(args):
@@ -195,16 +217,18 @@ def _process_args(args):
 
     if not sessions.session_manager.did_load:
         log.init.debug("Initializing main window...")
-        if config.val.content.private_browsing and qtutils.is_single_process():
+        private = args.target == 'private-window'
+        if (config.val.content.private_browsing or
+                private) and qtutils.is_single_process():
             err = Exception("Private windows are unavailable with "
                             "the single-process process model.")
             error.handle_fatal_exc(err, 'Cannot start in private mode',
                                    no_err_windows=args.no_err_windows)
             sys.exit(usertypes.Exit.err_init)
-        window = mainwindow.MainWindow(private=None)
+        window = mainwindow.MainWindow(private=private)
         if not args.nowindow:
             window.show()
-        q_app.setActiveWindow(window)
+        objects.qapp.setActiveWindow(window)
 
     process_pos_args(args.command)
     _open_startpage()
@@ -227,21 +251,32 @@ def process_pos_args(args, via_ipc=False, cwd=None, target_arg=None):
                     ipc. If the --target argument was not specified, target_arg
                     will be an empty string.
     """
-    if via_ipc and not args:
-        win_id = mainwindow.get_window(via_ipc, force_window=True)
+    new_window_target = ('private-window' if target_arg == 'private-window'
+                         else 'window')
+    command_target = config.val.new_instance_open_target
+    if command_target in {'window', 'private-window'}:
+        command_target = 'tab-silent'
+
+    win_id: Optional[int] = None
+
+    if via_ipc and (not args or args == ['']):
+        win_id = mainwindow.get_window(via_ipc=via_ipc,
+                                       target=new_window_target)
         _open_startpage(win_id)
         return
-    win_id = None
+
     for cmd in args:
         if cmd.startswith(':'):
             if win_id is None:
-                win_id = mainwindow.get_window(via_ipc, force_tab=True)
+                win_id = mainwindow.get_window(via_ipc=via_ipc,
+                                               target=command_target)
             log.init.debug("Startup cmd {!r}".format(cmd))
             commandrunner = runners.CommandRunner(win_id)
             commandrunner.run_safely(cmd[1:])
         elif not cmd:
             log.init.debug("Empty argument")
-            win_id = mainwindow.get_window(via_ipc, force_window=True)
+            win_id = mainwindow.get_window(via_ipc=via_ipc,
+                                           target=new_window_target)
         else:
             if via_ipc and target_arg and target_arg != 'auto':
                 open_target = target_arg
@@ -272,7 +307,7 @@ def open_url(url, target=None, no_raise=False, via_ipc=True):
     """
     target = target or config.val.new_instance_open_target
     background = target in {'tab-bg', 'tab-bg-silent'}
-    win_id = mainwindow.get_window(via_ipc, force_target=target,
+    win_id = mainwindow.get_window(via_ipc=via_ipc, target=target,
                                    no_raise=no_raise)
     tabbed_browser = objreg.get('tabbed-browser', scope='window',
                                 window=win_id)
@@ -291,7 +326,7 @@ def _open_startpage(win_id=None):
                 If set, open the startpage in the given window.
     """
     if win_id is not None:
-        window_ids = [win_id]  # type: typing.Iterable[int]
+        window_ids: Iterable[int] = [win_id]
     else:
         window_ids = objreg.window_registry
     for cur_win_id in list(window_ids):  # Copying as the dict could change
@@ -332,20 +367,48 @@ def _open_special_pages(args):
          objects.backend == usertypes.Backend.QtWebKit,
          'qute://warning/webkit'),
 
-        ('old-qt-warning-shown',
-         not qtutils.version_check('5.11'),
-         'qute://warning/old-qt'),
-
         ('session-warning-shown',
          qtutils.version_check('5.15', compiled=False),
          'qute://warning/sessions'),
     ]
 
+    if 'quickstart-done' not in general_sect:
+        # New users aren't going to be affected by the Qt 5.15 session change much, as
+        # they aren't used to qutebrowser saving the full back/forward history in
+        # sessions.
+        general_sect['session-warning-shown'] = '1'
+
     for state, condition, url in pages:
         if general_sect.get(state) != '1' and condition:
             tabbed_browser.tabopen(QUrl(url), background=False)
             general_sect[state] = '1'
 
+    # Show changelog on new releases
+    change = configfiles.state.qutebrowser_version_changed
+    if change == configfiles.VersionChange.equal:
+        return
+
+    setting = config.val.changelog_after_upgrade
+    if not change.matches_filter(setting):
+        log.init.debug(
+            f"Showing changelog is disabled (setting {setting}, change {change})")
+        return
+
+    try:
+        changelog = utils.read_file('html/doc/changelog.html')
+    except OSError as e:
+        log.init.warning(f"Not showing changelog due to {e}")
+        return
+
+    qbversion = qutebrowser.__version__
+    if f'id="v{qbversion}"' not in changelog:
+        log.init.warning("Not showing changelog (anchor not found)")
+        return
+
+    message.info(f"Showing changelog after upgrade to qutebrowser v{qbversion}.")
+    changelog_url = f'qute://help/changelog.html#v{qbversion}'
+    tabbed_browser.tabopen(QUrl(changelog_url), background=False)
+
 
 def on_focus_changed(_old, new):
     """Register currently focused main window in the object registry."""
@@ -367,14 +430,15 @@ def on_focus_changed(_old, new):
 
 def open_desktopservices_url(url):
     """Handler to open a URL via QDesktopServices."""
-    win_id = mainwindow.get_window(via_ipc=True, force_window=False)
+    target = config.val.new_instance_open_target
+    win_id = mainwindow.get_window(via_ipc=True, target=target)
     tabbed_browser = objreg.get('tabbed-browser', scope='window',
                                 window=win_id)
     tabbed_browser.tabopen(url)
 
 
 # This is effectively a @config.change_filter
-# Howerver, logging is initialized too early to use that annotation
+# However, logging is initialized too early to use that annotation
 def _on_config_changed(name: str) -> None:
     if name.startswith('logging.'):
         log.init_from_config(config.val)
@@ -391,7 +455,7 @@ def _init_modules(*, args):
     config.instance.changed.connect(_on_config_changed)
 
     log.init.debug("Initializing save manager...")
-    save_manager = savemanager.SaveManager(q_app)
+    save_manager = savemanager.SaveManager(objects.qapp)
     objreg.register('save-manager', save_manager)
     quitter.instance.shutting_down.connect(save_manager.shutdown)
     configinit.late_init(save_manager)
@@ -413,23 +477,23 @@ def _init_modules(*, args):
     downloads.init()
     quitter.instance.shutting_down.connect(downloads.shutdown)
 
-    try:
-        log.init.debug("Initializing SQL...")
-        sql.init(os.path.join(standarddir.data(), 'history.sqlite'))
+    with debug.log_time("init", "Initializing SQL/history"):
+        try:
+            log.init.debug("Initializing SQL...")
+            sql.init(os.path.join(standarddir.data(), 'history.sqlite'))
 
-        log.init.debug("Initializing web history...")
-        history.init(q_app)
-    except sql.KnownError as e:
-        error.handle_fatal_exc(e, 'Error initializing SQL',
-                               pre_text='Error initializing SQL',
-                               no_err_windows=args.no_err_windows)
-        sys.exit(usertypes.Exit.err_init)
+            log.init.debug("Initializing web history...")
+            history.init(objects.qapp)
+        except sql.KnownError as e:
+            error.handle_fatal_exc(e, 'Error initializing SQL',
+                                   pre_text='Error initializing SQL',
+                                   no_err_windows=args.no_err_windows)
+            sys.exit(usertypes.Exit.err_init)
 
     log.init.debug("Initializing command history...")
     cmdhistory.init()
     log.init.debug("Initializing sessions...")
-    sessions.init(q_app)
-    quitter.instance.shutting_down.connect(sessions.shutdown)
+    sessions.init(objects.qapp)
 
     log.init.debug("Initializing websettings...")
     websettings.init(args)
@@ -439,30 +503,28 @@ def _init_modules(*, args):
         crashsignal.crash_handler.display_faulthandler()
 
     log.init.debug("Initializing quickmarks...")
-    quickmark_manager = urlmarks.QuickmarkManager(q_app)
+    quickmark_manager = urlmarks.QuickmarkManager(objects.qapp)
     objreg.register('quickmark-manager', quickmark_manager)
 
     log.init.debug("Initializing bookmarks...")
-    bookmark_manager = urlmarks.BookmarkManager(q_app)
+    bookmark_manager = urlmarks.BookmarkManager(objects.qapp)
     objreg.register('bookmark-manager', bookmark_manager)
 
     log.init.debug("Initializing cookies...")
-    cookies.init(q_app)
+    cookies.init(objects.qapp)
 
     log.init.debug("Initializing cache...")
-    cache.init(q_app)
+    cache.init(objects.qapp)
 
     log.init.debug("Initializing downloads...")
-    download_manager = qtnetworkdownloads.DownloadManager(parent=q_app)
-    objreg.register('qtnetwork-download-manager', download_manager)
+    qtnetworkdownloads.init()
 
     log.init.debug("Initializing Greasemonkey...")
     greasemonkey.init()
 
     log.init.debug("Misc initialization...")
     macros.init()
-    # Init backend-specific stuff
-    browsertab.init()
+    windowundo.init()
 
 
 class Application(QApplication):
@@ -472,9 +534,14 @@ class Application(QApplication):
     Attributes:
         _args: ArgumentParser instance.
         _last_focus_object: The last focused object's repr.
+
+    Signals:
+        new_window: A new window was created.
+        window_closing: A window is being closed.
     """
 
     new_window = pyqtSignal(mainwindow.MainWindow)
+    window_closing = pyqtSignal(mainwindow.MainWindow)
 
     def __init__(self, args):
         """Constructor.
@@ -484,7 +551,7 @@ class Application(QApplication):
         """
         self._last_focus_object = None
 
-        qt_args = configinit.qt_args(args)
+        qt_args = qtargs.qt_args(args)
         log.init.debug("Commandline args: {}".format(sys.argv[1:]))
         log.init.debug("Parsed: {}".format(args))
         log.init.debug("Qt arguments: {}".format(qt_args[1:]))
@@ -497,7 +564,16 @@ class Application(QApplication):
         self.launch_time = datetime.datetime.now()
         self.focusObjectChanged.connect(  # type: ignore[attr-defined]
             self.on_focus_object_changed)
+
         self.setAttribute(Qt.AA_UseHighDpiPixmaps, True)
+        self.setAttribute(Qt.AA_MacDontSwapCtrlAndMeta, True)
+
+        self.new_window.connect(self._on_new_window)
+
+    @pyqtSlot(mainwindow.MainWindow)
+    def _on_new_window(self, window):
+        window.tabbed_browser.shutting_down.connect(functools.partial(
+            self.window_closing.emit, window))
 
     @pyqtSlot(QObject)
     def on_focus_object_changed(self, obj):
@@ -509,15 +585,15 @@ class Application(QApplication):
 
     def event(self, e):
         """Handle macOS FileOpen events."""
-        if e.type() == QEvent.FileOpen:
-            url = e.url()
-            if url.isValid():
-                open_url(url, no_raise=True)
-            else:
-                message.error("Invalid URL: {}".format(url.errorString()))
-        else:
+        if e.type() != QEvent.FileOpen:
             return super().event(e)
 
+        url = e.url()
+        if url.isValid():
+            open_url(url, no_raise=True)
+        else:
+            message.error("Invalid URL: {}".format(url.errorString()))
+
         return True
 
     def __repr__(self):
