@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2015-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2015-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -15,11 +15,11 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <http://www.gnu.org/licenses/>.
+# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
 
 """Utilities related to QWebHistory."""
 
-import typing
+from typing import Any, List, Mapping
 
 from PyQt5.QtCore import QByteArray, QDataStream, QIODevice, QUrl
 
@@ -81,7 +81,7 @@ def serialize(items):
     """
     data = QByteArray()
     stream = QDataStream(data, QIODevice.ReadWrite)
-    user_data = []  # type: typing.List[typing.Mapping[str, typing.Any]]
+    user_data: List[Mapping[str, Any]] = []
 
     current_idx = None
 
