@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2016-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2016-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -15,7 +15,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <http://www.gnu.org/licenses/>.
+# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
 
 """Showing prompts above the statusbar."""
 
@@ -23,9 +23,9 @@ import os.path
 import html
 import collections
 import functools
-import typing
+import dataclasses
+from typing import Deque, MutableSequence, Optional, cast
 
-import attr
 from PyQt5.QtCore import (pyqtSlot, pyqtSignal, Qt, QTimer, QDir, QModelIndex,
                           QItemSelectionModel, QObject, QEventLoop)
 from PyQt5.QtWidgets import (QWidget, QGridLayout, QVBoxLayout, QLineEdit,
@@ -40,16 +40,16 @@ from qutebrowser.api import cmdutils
 from qutebrowser.utils import urlmatch
 
 
-prompt_queue = typing.cast('PromptQueue', None)
+prompt_queue = cast('PromptQueue', None)
 
 
-@attr.s
+@dataclasses.dataclass
 class AuthInfo:
 
     """Authentication info returned by a prompt."""
 
-    user = attr.ib()
-    password = attr.ib()
+    user: str
+    password: str
 
 
 class Error(Exception):
@@ -102,9 +102,8 @@ class PromptQueue(QObject):
         super().__init__(parent)
         self._question = None
         self._shutting_down = False
-        self._loops = []  # type: typing.MutableSequence[qtutils.EventLoop]
-        self._queue = collections.deque(
-        )  # type: typing.Deque[usertypes.Question]
+        self._loops: MutableSequence[qtutils.EventLoop] = []
+        self._queue: Deque[usertypes.Question] = collections.deque()
         message.global_bridge.mode_left.connect(self._on_mode_left)
 
     def __repr__(self):
@@ -192,13 +191,14 @@ class PromptQueue(QObject):
         if blocking:
             loop = qtutils.EventLoop()
             self._loops.append(loop)
-            loop.destroyed.connect(  # type: ignore
-                lambda: self._loops.remove(loop))
+            loop.destroyed.connect(lambda: self._loops.remove(loop))
             question.completed.connect(loop.quit)
             question.completed.connect(loop.deleteLater)
-            log.prompt.debug("Starting loop.exec_() for {}".format(question))
-            loop.exec_(QEventLoop.ExcludeSocketNotifiers)
-            log.prompt.debug("Ending loop.exec_() for {}".format(question))
+            log.prompt.debug("Starting loop.exec() for {}".format(question))
+            flags = cast(QEventLoop.ProcessEventsFlags,
+                         QEventLoop.ExcludeSocketNotifiers)
+            loop.exec(flags)
+            log.prompt.debug("Ending loop.exec() for {}".format(question))
 
             log.prompt.debug("Restoring old question {}".format(old_question))
             self._question = old_question
@@ -288,7 +288,7 @@ class PromptContainer(QWidget):
         self._layout = QVBoxLayout(self)
         self._layout.setContentsMargins(10, 10, 10, 10)
         self._win_id = win_id
-        self._prompt = None  # type: typing.Optional[_BasePrompt]
+        self._prompt: Optional[_BasePrompt] = None
 
         self.setObjectName('PromptContainer')
         self.setAttribute(Qt.WA_StyledBackground, True)
@@ -329,7 +329,7 @@ class PromptContainer(QWidget):
             usertypes.PromptMode.alert: AlertPrompt,
         }
         klass = classes[question.mode]
-        prompt = typing.cast(_BasePrompt, klass(question))
+        prompt = klass(question)
 
         log.prompt.debug("Displaying prompt {}".format(prompt))
         self._prompt = prompt
@@ -590,6 +590,7 @@ class LineEditPrompt(_BasePrompt):
         self._vbox.addWidget(self._lineedit)
         if question.default:
             self._lineedit.setText(question.default)
+            self._lineedit.selectAll()
         self.setFocusProxy(self._lineedit)
         self._init_key_label()
 
@@ -600,7 +601,7 @@ class LineEditPrompt(_BasePrompt):
         return True
 
     def _allowed_commands(self):
-        return [('prompt-accept', 'Accept'), ('leave-mode', 'Abort')]
+        return [('prompt-accept', 'Accept'), ('mode-leave', 'Abort')]
 
 
 class FilenamePrompt(_BasePrompt):
@@ -707,7 +708,7 @@ class FilenamePrompt(_BasePrompt):
         # Nothing selected initially
         self._file_view.setCurrentIndex(QModelIndex())
         # The model needs to be sorted so we get the correct first/last index
-        self._file_model.directoryLoaded.connect(  # type: ignore
+        self._file_model.directoryLoaded.connect(
             lambda: self._file_model.sort(0))
 
     def accept(self, value=None, save=False):
@@ -755,7 +756,7 @@ class FilenamePrompt(_BasePrompt):
 
         selmodel.setCurrentIndex(
             idx,
-            QItemSelectionModel.ClearAndSelect |  # type: ignore
+            QItemSelectionModel.ClearAndSelect |  # type: ignore[arg-type]
             QItemSelectionModel.Rows)
         self._insert_path(idx, clicked=False)
 
@@ -772,7 +773,7 @@ class FilenamePrompt(_BasePrompt):
         return idx
 
     def _allowed_commands(self):
-        return [('prompt-accept', 'Accept'), ('leave-mode', 'Abort')]
+        return [('prompt-accept', 'Accept'), ('mode-leave', 'Abort')]
 
 
 class DownloadFilenamePrompt(FilenamePrompt):
@@ -782,7 +783,7 @@ class DownloadFilenamePrompt(FilenamePrompt):
     def __init__(self, question, parent=None):
         super().__init__(question, parent)
         self._file_model.setFilter(
-            QDir.AllDirs | QDir.Drives | QDir.NoDot)  # type: ignore
+            QDir.AllDirs | QDir.Drives | QDir.NoDot)  # type: ignore[arg-type]
 
     def accept(self, value=None, save=False):
         done = super().accept(value, save)
@@ -793,8 +794,7 @@ class DownloadFilenamePrompt(FilenamePrompt):
 
     def download_open(self, cmdline, pdfjs):
         if pdfjs:
-            target = downloads.PDFJSDownloadTarget(
-            )  # type: downloads._DownloadTarget
+            target: 'downloads._DownloadTarget' = downloads.PDFJSDownloadTarget()
         else:
             target = downloads.OpenFileDownloadTarget(cmdline)
 
@@ -805,7 +805,7 @@ class DownloadFilenamePrompt(FilenamePrompt):
     def _allowed_commands(self):
         cmds = [
             ('prompt-accept', 'Accept'),
-            ('leave-mode', 'Abort'),
+            ('mode-leave', 'Abort'),
             ('prompt-open-download', "Open download"),
             ('prompt-open-download --pdfjs', "Open download via PDF.js"),
             ('prompt-yank', "Yank URL"),
@@ -869,7 +869,7 @@ class AuthenticationPrompt(_BasePrompt):
 
     def _allowed_commands(self):
         return [('prompt-accept', "Accept"),
-                ('leave-mode', "Abort")]
+                ('mode-leave', "Abort")]
 
 
 class YesNoPrompt(_BasePrompt):
@@ -931,7 +931,7 @@ class YesNoPrompt(_BasePrompt):
             default = 'yes' if self.question.default else 'no'
             cmds.append(('prompt-accept', "Use default ({})".format(default)))
 
-        cmds.append(('leave-mode', "Abort"))
+        cmds.append(('mode-leave', "Abort"))
         cmds.append(('prompt-yank', "Yank URL"))
         return cmds
 
@@ -960,5 +960,5 @@ def init():
     """Initialize global prompt objects."""
     global prompt_queue
     prompt_queue = PromptQueue()
-    message.global_bridge.ask_question.connect(  # type: ignore
+    message.global_bridge.ask_question.connect(  # type: ignore[call-arg]
         prompt_queue.ask_question, Qt.DirectConnection)
