@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -15,7 +15,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <http://www.gnu.org/licenses/>.
+# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
 
 """The commandline in the statusbar."""
 
@@ -71,9 +71,8 @@ class Command(misc.MinimalLineEditMixin, misc.CommandLineEdit):
             self.history.changed.connect(command_history.changed)
         self.setSizePolicy(QSizePolicy.MinimumExpanding, QSizePolicy.Ignored)
 
-        self.cursorPositionChanged.connect(
-            self.update_completion)  # type: ignore
-        self.textChanged.connect(self.update_completion)  # type: ignore
+        self.cursorPositionChanged.connect(self.update_completion)
+        self.textChanged.connect(self.update_completion)
         self.textChanged.connect(self.updateGeometry)
         self.textChanged.connect(self._incremental_search)
 
@@ -148,7 +147,7 @@ class Command(misc.MinimalLineEditMixin, misc.CommandLineEdit):
             raise cmdutils.CommandError(
                 "Invalid command text '{}'.".format(text))
         if run_on_count and count is not None:
-            self.got_cmd[str, int].emit(text, count)  # type: ignore
+            self.got_cmd[str, int].emit(text, count)
         else:
             self.set_cmd_text(text)
 
@@ -198,7 +197,7 @@ class Command(misc.MinimalLineEditMixin, misc.CommandLineEdit):
                           'cmd accept')
 
         if not was_search:
-            self.got_cmd[str].emit(text[1:])  # type: ignore
+            self.got_cmd[str].emit(text[1:])
 
     @cmdutils.register(instance='status-command', scope='window')
     def edit_command(self, run: bool = False) -> None:
