@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -15,16 +15,17 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <http://www.gnu.org/licenses/>.
+# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
 
 """Utilities related to jinja2."""
 
 import os
 import os.path
-import typing
+import posixpath
 import functools
 import contextlib
 import html
+from typing import Any, Callable, FrozenSet, Iterator, List, Set, Tuple
 
 import jinja2
 import jinja2.nodes
@@ -68,7 +69,7 @@ class Loader(jinja2.BaseLoader):
             self,
             _env: jinja2.Environment,
             template: str
-    ) -> typing.Tuple[str, str, typing.Callable[[], bool]]:
+    ) -> Tuple[str, str, Callable[[], bool]]:
         path = os.path.join(self._subdir, template)
         try:
             source = utils.read_file(path)
@@ -98,31 +99,32 @@ class Environment(jinja2.Environment):
         self._autoescape = True
 
     @contextlib.contextmanager
-    def no_autoescape(self) -> typing.Iterator[None]:
+    def no_autoescape(self) -> Iterator[None]:
         """Context manager to temporarily turn off autoescaping."""
         self._autoescape = False
         yield
         self._autoescape = True
 
     def _resource_url(self, path: str) -> str:
-        """Load images from a relative path (to qutebrowser).
+        """Load qutebrowser resource files.
 
         Arguments:
-            path: The relative path to the image
+            path: The relative path to the resource.
         """
-        image = utils.resource_filename(path)
-        url = QUrl.fromLocalFile(image)
-        urlstr = url.toString(QUrl.FullyEncoded)  # type: ignore
+        assert not posixpath.isabs(path), path
+        url = QUrl('qute://resource')
+        url.setPath('/' + path)
+        urlutils.ensure_valid(url)
+        urlstr = url.toString(QUrl.FullyEncoded)  # type: ignore[arg-type]
         return urlstr
 
     def _data_url(self, path: str) -> str:
         """Get a data: url for the broken qutebrowser logo."""
-        data = utils.read_file(path, binary=True)
-        filename = utils.resource_filename(path)
-        mimetype = utils.guess_mimetype(filename)
+        data = utils.read_file_binary(path)
+        mimetype = utils.guess_mimetype(path)
         return urlutils.data_url(mimetype, data).toString()
 
-    def getattr(self, obj: typing.Any, attribute: str) -> typing.Any:
+    def getattr(self, obj: Any, attribute: str) -> Any:
         """Override jinja's getattr() to be less clever.
 
         This means it doesn't fall back to __getitem__, and it doesn't hide
@@ -131,7 +133,7 @@ class Environment(jinja2.Environment):
         return getattr(obj, attribute)
 
 
-def render(template: str, **kwargs: typing.Any) -> str:
+def render(template: str, **kwargs: Any) -> str:
     """Render the given template and pass the given arguments to it."""
     return environment.get_template(template).render(**kwargs)
 
@@ -142,10 +144,10 @@ js_environment = jinja2.Environment(loader=Loader('javascript'))
 
 @debugcachestats.register()
 @functools.lru_cache()
-def template_config_variables(template: str) -> typing.FrozenSet[str]:
+def template_config_variables(template: str) -> FrozenSet[str]:
     """Return the config variables used in the template."""
     unvisted_nodes = [environment.parse(template)]
-    result = set()  # type: typing.Set[str]
+    result: Set[str] = set()
     while unvisted_nodes:
         node = unvisted_nodes.pop()
         if not isinstance(node, jinja2.nodes.Getattr):
@@ -154,13 +156,13 @@ def template_config_variables(template: str) -> typing.FrozenSet[str]:
 
         # List of attribute names in reverse order.
         # For example it's ['ab', 'c', 'd'] for 'conf.d.c.ab'.
-        attrlist = []  # type: typing.List[str]
+        attrlist: List[str] = []
         while isinstance(node, jinja2.nodes.Getattr):
-            attrlist.append(node.attr)  # type: ignore
-            node = node.node  # type: ignore
+            attrlist.append(node.attr)  # type: ignore[attr-defined]
+            node = node.node  # type: ignore[attr-defined]
 
         if isinstance(node, jinja2.nodes.Name):
-            if node.name == 'conf':  # type: ignore
+            if node.name == 'conf':  # type: ignore[attr-defined]
                 result.add('.'.join(reversed(attrlist)))
             # otherwise, the node is a Name node so it doesn't have any
             # child nodes
