@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2015-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2015-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -15,15 +15,15 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <http://www.gnu.org/licenses/>.
+# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
 
 """Base class for a subprocess run for tests."""
 
 import re
-import os
 import time
+import warnings
+import dataclasses
 
-import attr
 import pytest
 import pytestqt.plugin
 from PyQt5.QtCore import (pyqtSlot, pyqtSignal, QProcess, QObject,
@@ -55,7 +55,7 @@ class BlacklistedMessageError(Exception):
     """Raised when ensure_not_logged found a message."""
 
 
-@attr.s
+@dataclasses.dataclass
 class Line:
 
     """Container for a line of data the process emits.
@@ -65,8 +65,8 @@ class Line:
         waited_for: If Process.wait_for was used on this line already.
     """
 
-    data = attr.ib()
-    waited_for = attr.ib(False)
+    data: str
+    waited_for: bool = False
 
 
 def _render_log(data, *, verbose, threshold=100):
@@ -100,7 +100,7 @@ def pytest_runtest_makereport(item, call):
         return
 
     quteproc_log = getattr(item, '_quteproc_log', None)
-    server_log = getattr(item, '_server_log', None)
+    server_logs = getattr(item, '_server_logs', [])
 
     if not hasattr(report.longrepr, 'addsection'):
         # In some conditions (on macOS and Windows it seems), report.longrepr
@@ -113,11 +113,11 @@ def pytest_runtest_makereport(item, call):
 
     verbose = item.config.getoption('--verbose')
     if quteproc_log is not None:
-        report.longrepr.addsection("qutebrowser output",
-                                   _render_log(quteproc_log, verbose=verbose))
-    if server_log is not None:
-        report.longrepr.addsection("server output",
-                                   _render_log(server_log, verbose=verbose))
+        report.longrepr.addsection(
+            "qutebrowser output", _render_log(quteproc_log, verbose=verbose))
+    for name, content in server_logs:
+        report.longrepr.addsection(
+            f"{name} output", _render_log(content, verbose=verbose))
 
 
 class Process(QObject):
@@ -233,7 +233,7 @@ class Process(QObject):
         self._started = True
         verbose = self.request.config.getoption('--verbose')
 
-        timeout = 60 if 'CI' in os.environ else 20
+        timeout = 60 if utils.ON_CI else 20
         for _ in range(timeout):
             with self._wait_signal(self.ready, timeout=1000,
                                    raising=False) as blocker:
@@ -316,8 +316,11 @@ class Process(QObject):
         else:
             self.proc.terminate()
 
-        ok = self.proc.waitForFinished()
+        ok = self.proc.waitForFinished(5000)
         if not ok:
+            cmdline = ' '.join([self.proc.program()] + self.proc.arguments())
+            warnings.warn(f"Test process {cmdline} with PID {self.proc.processId()} "
+                          "failed to terminate!")
             self.proc.kill()
             self.proc.waitForFinished()
 
@@ -472,7 +475,7 @@ class Process(QObject):
         if timeout is None:
             if do_skip:
                 timeout = 2000
-            elif 'CI' in os.environ:
+            elif utils.ON_CI:
                 timeout = 15000
             else:
                 timeout = 5000
@@ -512,3 +515,4 @@ class Process(QObject):
         self.exit_expected = True
         with self._wait_signal(self.proc.finished, timeout=15000):
             pass
+        assert not self.is_running()
