@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2015-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2015-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -15,7 +15,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <http://www.gnu.org/licenses/>.
+# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
 
 """Tests for browser.network.networkreply."""
 
@@ -52,9 +52,8 @@ class TestFixedDataNetworkReply:
                                       b'Hello World! This is a test.'])
     def test_data(self, qtbot, req, data):
         reply = networkreply.FixedDataNetworkReply(req, data, 'test/foo')
-        with qtbot.waitSignal(reply.metaDataChanged), \
-             qtbot.waitSignal(reply.readyRead), \
-             qtbot.waitSignal(reply.finished):
+        with qtbot.waitSignals([reply.metaDataChanged, reply.readyRead,
+                                reply.finished], order='strict'):
             pass
 
         assert reply.bytesAvailable() == len(data)
@@ -79,7 +78,7 @@ def test_error_network_reply(qtbot, req):
     reply = networkreply.ErrorNetworkReply(
         req, "This is an error", QNetworkReply.UnknownNetworkError)
 
-    with qtbot.waitSignal(reply.error), qtbot.waitSignal(reply.finished):
+    with qtbot.waitSignals([reply.error, reply.finished], order='strict'):
         pass
 
     reply.abort()  # shouldn't do anything
