@@ -1,5 +1,5 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
-# Copyright 2014-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 
 # This file is part of qutebrowser.
 #
@@ -14,7 +14,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <http://www.gnu.org/licenses/>.
+# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
 
 """Tests for qutebrowser.config.configcommands."""
 
@@ -212,14 +212,12 @@ class TestSet:
             commands.set(win_id=0, option='foo?')
 
 
-@pytest.mark.parametrize('old', [True, False])
-def test_diff(commands, tabbed_browser_stubs, old):
+def test_diff(commands, tabbed_browser_stubs):
     """Run ':config-diff'.
 
     Should open qute://configdiff."""
-    commands.config_diff(win_id=0, old=old)
-    url = QUrl('qute://configdiff/old') if old else QUrl('qute://configdiff')
-    assert tabbed_browser_stubs[0].loaded_url == url
+    commands.config_diff(win_id=0)
+    assert tabbed_browser_stubs[0].loaded_url == QUrl('qute://configdiff')
 
 
 class TestCycle:
@@ -299,7 +297,7 @@ class TestAdd:
     @pytest.mark.parametrize('temp', [True, False])
     @pytest.mark.parametrize('value', ['test1', 'test2'])
     def test_list_add(self, commands, config_stub, yaml_value, temp, value):
-        name = 'content.host_blocking.whitelist'
+        name = 'content.blocking.whitelist'
 
         commands.config_list_add(name, value, temp=temp)
 
@@ -326,7 +324,7 @@ class TestAdd:
         with pytest.raises(
                 cmdutils.CommandError,
                 match="Invalid value '{}'".format(value)):
-            commands.config_list_add('content.host_blocking.whitelist', value)
+            commands.config_list_add('content.blocking.whitelist', value)
 
     @pytest.mark.parametrize('value', ['test1', 'test2'])
     @pytest.mark.parametrize('temp', [True, False])
@@ -499,7 +497,8 @@ class TestSource:
         else:
             assert False, location
 
-        pyfile.write_text('c.content.javascript.enabled = False\n',
+        pyfile.write_text('\n'.join(['config.load_autoconfig(False)',
+                                     'c.content.javascript.enabled = False']),
                           encoding='utf-8')
 
         commands.config_source(arg, clear=clear)
@@ -511,14 +510,16 @@ class TestSource:
 
     def test_config_py_arg_source(self, commands, config_py_arg, config_stub):
         assert config_stub.val.content.javascript.enabled
-        config_py_arg.write_text('c.content.javascript.enabled = False\n',
+        config_py_arg.write_text('\n'.join(['config.load_autoconfig(False)',
+                                            'c.content.javascript.enabled = False']),
                                  encoding='utf-8')
         commands.config_source()
         assert not config_stub.val.content.javascript.enabled
 
     def test_errors(self, commands, config_tmpdir):
         pyfile = config_tmpdir / 'config.py'
-        pyfile.write_text('c.foo = 42', encoding='utf-8')
+        pyfile.write_text('\n'.join(['config.load_autoconfig(False)',
+                                     'c.foo = 42']), encoding='utf-8')
 
         with pytest.raises(cmdutils.CommandError) as excinfo:
             commands.config_source()
@@ -529,7 +530,8 @@ class TestSource:
 
     def test_invalid_source(self, commands, config_tmpdir):
         pyfile = config_tmpdir / 'config.py'
-        pyfile.write_text('1/0', encoding='utf-8')
+        pyfile.write_text('\n'.join(['config.load_autoconfig(False)',
+                                     '1/0']), encoding='utf-8')
 
         with pytest.raises(cmdutils.CommandError) as excinfo:
             commands.config_source()
@@ -571,7 +573,8 @@ class TestEdit:
 
     def test_with_sourcing(self, commands, config_stub, patch_editor):
         assert config_stub.val.content.javascript.enabled
-        mock = patch_editor('c.content.javascript.enabled = False')
+        mock = patch_editor('\n'.join(['config.load_autoconfig(False)',
+                                       'c.content.javascript.enabled = False']))
 
         commands.config_edit()
 
@@ -580,16 +583,16 @@ class TestEdit:
 
     def test_config_py_with_sourcing(self, commands, config_stub, patch_editor, config_py_arg):
         assert config_stub.val.content.javascript.enabled
-        conf = 'c.content.javascript.enabled = False'
-        mock = patch_editor(conf)
+        conf = ['config.load_autoconfig(False)', 'c.content.javascript.enabled = False']
+        mock = patch_editor("\n".join(conf))
         commands.config_edit()
         mock.assert_called_once_with(unittest.mock.ANY)
         assert not config_stub.val.content.javascript.enabled
-        assert config_py_arg.read_text('utf-8').splitlines() == [conf]
+        assert config_py_arg.read_text('utf-8').splitlines() == conf
 
     def test_error(self, commands, config_stub, patch_editor, message_mock,
                    caplog):
-        patch_editor('c.foo = 42')
+        patch_editor('\n'.join(['config.load_autoconfig(False)', 'c.foo = 42']))
 
         with caplog.at_level(logging.ERROR):
             commands.config_edit()
