@@ -1,5 +1,5 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
-# Copyright 2014-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 
 # This file is part of qutebrowser.
 #
@@ -14,18 +14,19 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <http://www.gnu.org/licenses/>.
+# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
 
 """Tests for qutebrowser.config.configtypes."""
 
 import re
+import sys
 import json
 import math
 import warnings
 import inspect
 import functools
+import dataclasses
 
-import attr
 import pytest
 import hypothesis
 from hypothesis import strategies
@@ -137,10 +138,16 @@ class TestValidValues:
 
     def test_descriptions(self, klass):
         """Test descriptions."""
-        vv = klass(('foo', "foo desc"), ('bar', "bar desc"), 'baz')
-        assert vv.descriptions['foo'] == "foo desc"
-        assert vv.descriptions['bar'] == "bar desc"
-        assert 'baz' not in vv.descriptions
+        vv = klass(
+            ('one-with', "desc 1"),
+            ('two-with', "desc 2"),
+            'three-without',
+            ('four-without', None)
+        )
+        assert vv.descriptions['one-with'] == "desc 1"
+        assert vv.descriptions['two-with'] == "desc 2"
+        assert 'three-without' not in vv.descriptions
+        assert 'four-without' not in vv.descriptions
 
     @pytest.mark.parametrize('args, expected', [
         (['a', 'b'], "<qutebrowser.config.configtypes.ValidValues "
@@ -248,10 +255,11 @@ class TestAll:
                 configtypes.PercOrInt,  # ditto
         ]:
             return
-        elif (isinstance(klass, functools.partial) and
-              klass.func in [configtypes.ListOrValue, configtypes.List]):
+        elif (isinstance(klass, functools.partial) and klass.func in [
+                configtypes.ListOrValue, configtypes.List, configtypes.Dict]):
             # ListOrValue: "- /" -> "/"
             # List: "- /" -> ["/"]
+            # Dict: '{":": "A"}' -> ':: A'
             return
 
         assert converted == s
@@ -303,6 +311,18 @@ class TestAll:
             for value, _desc in completions:
                 typ.from_str(value)
 
+    def test_custom_completions(self, klass):
+        """Make sure we can pass custom completions."""
+        completions = [('1', 'one'), ('2', 'two')]
+        typ = klass(completions=completions)
+        assert typ.complete() == completions
+
+    def test_signature(self, klass):
+        """Make sure flag arguments are kw-only."""
+        sig = inspect.signature(klass)
+        for name in ['none_ok', 'completions']:
+            assert sig.parameters[name].kind == inspect.Parameter.KEYWORD_ONLY
+
 
 class TestBaseType:
 
@@ -394,14 +414,11 @@ class MappingSubclass(configtypes.MappingType):
     """A MappingType we use in TestMappingType which is valid/good."""
 
     MAPPING = {
-        'one': 1,
-        'two': 2,
+        'one': (1, 'one doc'),
+        'two': (2, 'two doc'),
+        'three': (3, None),
     }
 
-    def __init__(self, none_ok=False):
-        super().__init__(none_ok)
-        self.valid_values = configtypes.ValidValues('one', 'two')
-
 
 class TestMappingType:
 
@@ -427,11 +444,12 @@ class TestMappingType:
     def test_to_str(self, klass):
         assert klass().to_str('one') == 'one'
 
-    @pytest.mark.parametrize('typ', [configtypes.ColorSystem(),
-                                     configtypes.Position(),
-                                     configtypes.SelectOnRemove()])
-    def test_mapping_type_matches_valid_values(self, typ):
-        assert sorted(typ.MAPPING) == sorted(typ.valid_values)
+    def test_valid_values(self, klass):
+        assert klass().valid_values == configtypes.ValidValues(
+            ('one', 'one doc'),
+            ('two', 'two doc'),
+            ('three', None),
+        )
 
 
 class TestString:
@@ -528,8 +546,7 @@ class ListSubclass(configtypes.List):
             elemtype = configtypes.String(none_ok=none_ok_inner)
         super().__init__(elemtype, none_ok=none_ok_outer, length=length)
         if set_valid_values:
-            self.valtype.valid_values = configtypes.ValidValues(
-                'foo', 'bar', 'baz')
+            self.valtype.valid_values = configtypes.ValidValues('foo', 'bar', 'baz')
 
 
 class FlagListSubclass(configtypes.FlagList):
@@ -1344,14 +1361,14 @@ class TestQssColor:
             klass().to_py(val)
 
 
-@attr.s
+@dataclasses.dataclass
 class FontDesc:
 
-    style = attr.ib()
-    weight = attr.ib()
-    pt = attr.ib()
-    px = attr.ib()
-    family = attr.ib()
+    style: QFont.Style
+    weight: QFont.Weight
+    pt: int
+    px: int
+    family: str
 
 
 class TestFont:
@@ -1482,27 +1499,19 @@ class TestRegex:
     @pytest.mark.parametrize('val', [
         pytest.param(r'(foo|bar))?baz[fis]h', id='unmatched parens'),
         pytest.param('(' * 500, id='too many parens'),
+        pytest.param(r'foo\Xbar', id='invalid escape X'),
+        pytest.param(r'foo\Cbar', id='invalid escape C'),
+        pytest.param(r'[[]]', id='nested set', marks=pytest.mark.skipif(
+            sys.hexversion < 0x03070000,
+            reason="Warning was added in Python 3.7")),
+        pytest.param(r'[a||b]', id='set operation', marks=pytest.mark.skipif(
+            sys.hexversion < 0x03070000,
+            reason="Warning was added in Python 3.7")),
     ])
     def test_to_py_invalid(self, klass, val):
         with pytest.raises(configexc.ValidationError):
             klass().to_py(val)
 
-    @pytest.mark.parametrize('val', [
-        r'foo\Xbar',
-        r'foo\Cbar',
-    ])
-    def test_to_py_maybe_valid(self, klass, val):
-        """Those values are valid on some Python versions (and systems?).
-
-        On others, they raise a DeprecationWarning because of an invalid
-        escape. This tests makes sure this gets translated to a
-        ValidationError.
-        """
-        try:
-            klass().to_py(val)
-        except configexc.ValidationError:
-            pass
-
     @pytest.mark.parametrize('warning', [
         Warning('foo'), DeprecationWarning('foo'),
     ])
@@ -1518,20 +1527,6 @@ class TestRegex:
         with pytest.raises(type(warning)):
             regex.to_py('foo')
 
-    def test_bad_pattern_warning(self, mocker, klass):
-        """Test a simulated bad pattern warning.
-
-        This only seems to happen with Python 3.5, so we simulate this for
-        better coverage.
-        """
-        regex = klass()
-        m = mocker.patch('qutebrowser.config.configtypes.re')
-        m.compile.side_effect = lambda *args: warnings.warn(r'bad escape \C',
-                                                            DeprecationWarning)
-        m.error = re.error
-        with pytest.raises(configexc.ValidationError):
-            regex.to_py('foo')
-
     @pytest.mark.parametrize('flags, expected', [
         (None, 0),
         ('IGNORECASE', re.IGNORECASE),
@@ -1969,7 +1964,7 @@ class TestFuzzyUrl:
         assert klass().to_py(val) == expected
 
     @pytest.mark.parametrize('val', [
-        '::foo',  # invalid URL
+        '',  # invalid URL
         'foo bar',  # invalid search term
     ])
     def test_to_py_invalid(self, klass, val):
