@@ -1,5 +1,5 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
-# Copyright 2017-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2017-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 
 # This file is part of qutebrowser.
 #
@@ -14,7 +14,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <http://www.gnu.org/licenses/>.
+# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
 
 """Tests for qutebrowser.browser.greasemonkey."""
 
@@ -27,6 +27,7 @@ from PyQt5.QtCore import QUrl
 
 from qutebrowser.utils import usertypes
 from qutebrowser.browser import greasemonkey
+from qutebrowser.misc import objects
 
 test_gm_script = r"""
 // ==UserScript==
@@ -40,12 +41,15 @@ test_gm_script = r"""
 console.log("Script is running.");
 """
 
-pytestmark = pytest.mark.usefixtures('data_tmpdir')
+pytestmark = [
+    pytest.mark.usefixtures('data_tmpdir'),
+    pytest.mark.usefixtures('config_tmpdir')
+]
 
 
 def _save_script(script_text, filename):
     # pylint: disable=no-member
-    file_path = py.path.local(greasemonkey._scripts_dir()) / filename
+    file_path = py.path.local(greasemonkey._scripts_dirs()[0]) / filename
     # pylint: enable=no-member
     file_path.write_text(script_text, encoding='utf-8', ensure=True)
 
@@ -168,15 +172,6 @@ def test_utf8_bom():
 
 class TestForceDocumentEnd:
 
-    @pytest.fixture
-    def patch(self, monkeypatch):
-        def _patch(*, backend, qt_512):
-            monkeypatch.setattr(greasemonkey.objects, 'backend', backend)
-            monkeypatch.setattr(greasemonkey.qtutils, 'version_check',
-                                lambda version, exact=False, compiled=True:
-                                qt_512)
-        return _patch
-
     def _get_script(self, *, namespace, name):
         source = textwrap.dedent("""
             // ==UserScript==
@@ -192,26 +187,15 @@ class TestForceDocumentEnd:
         assert len(scripts) == 1
         return scripts[0]
 
-    @pytest.mark.parametrize('backend, qt_512', [
-        (usertypes.Backend.QtWebKit, True),
-        (usertypes.Backend.QtWebEngine, False),
-    ])
-    def test_not_applicable(self, patch, backend, qt_512):
-        """Test backend/Qt version combinations which don't need a fix."""
-        patch(backend=backend, qt_512=qt_512)
-        script = self._get_script(namespace='https://github.com/ParticleCore',
-                                  name='Iridium')
-        assert not script.needs_document_end_workaround()
-
     @pytest.mark.parametrize('namespace, name, force', [
         ('http://userstyles.org', 'foobar', True),
         ('https://github.com/ParticleCore', 'Iridium', True),
         ('https://github.com/ParticleCore', 'Foo', False),
         ('https://example.org', 'Iridium', False),
     ])
-    def test_matching(self, patch, namespace, name, force):
+    def test_matching(self, monkeypatch, namespace, name, force):
         """Test matching based on namespace/name."""
-        patch(backend=usertypes.Backend.QtWebEngine, qt_512=True)
+        monkeypatch.setattr(objects, 'backend', usertypes.Backend.QtWebEngine)
         script = self._get_script(namespace=namespace, name=name)
         assert script.needs_document_end_workaround() == force
 
