@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2018-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2018-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -15,19 +15,20 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <http://www.gnu.org/licenses/>.
+# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
 
 # pylint: disable=line-too-long
 
 
 """Data used by test_keyutils.py to test all keys."""
 
+import dataclasses
+from typing import Optional
 
-import attr
 from PyQt5.QtCore import Qt
 
 
-@attr.s
+@dataclasses.dataclass(order=True)
 class Key:
 
     """A key with expected values.
@@ -40,21 +41,21 @@ class Key:
         member: The numeric value.
     """
 
-    attribute = attr.ib()
-    name = attr.ib(None)
-    text = attr.ib('')
-    uppertext = attr.ib('')
-    member = attr.ib(None)
-    qtest = attr.ib(True)
+    attribute: str
+    name: Optional[str] = None
+    text: str = ''
+    uppertext: str = ''
+    member: Optional[int] = None
+    qtest: bool = True
 
-    def __attrs_post_init__(self):
+    def __post_init__(self):
         if self.attribute:
             self.member = getattr(Qt, 'Key_' + self.attribute, None)
         if self.name is None:
             self.name = self.attribute
 
 
-@attr.s
+@dataclasses.dataclass(order=True)
 class Modifier:
 
     """A modifier with expected values.
@@ -66,11 +67,11 @@ class Modifier:
         member: The numeric value.
     """
 
-    attribute = attr.ib()
-    name = attr.ib(None)
-    member = attr.ib(None)
+    attribute: str
+    name: Optional[str] = None
+    member: Optional[int] = None
 
-    def __attrs_post_init__(self):
+    def __post_init__(self):
         self.member = getattr(Qt, self.attribute + 'Modifier')
         if self.name is None:
             self.name = self.attribute
