@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -15,7 +15,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <http://www.gnu.org/licenses/>.
+# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
 
 import operator
 
@@ -28,7 +28,6 @@ from PyQt5.QtWidgets import QWidget
 
 from unit.keyinput import key_data
 from qutebrowser.keyinput import keyutils
-from qutebrowser.utils import utils
 
 
 @pytest.fixture(params=key_data.KEYS, ids=lambda k: k.attribute)
@@ -268,6 +267,7 @@ class TestKeySequence:
     @pytest.mark.parametrize('orig, normalized', [
         ('<Control+x>', '<Ctrl+x>'),
         ('<Windows+x>', '<Meta+x>'),
+        ('<Super+x>', '<Meta+x>'),
         ('<Mod4+x>', '<Meta+x>'),
         ('<Command+x>', '<Meta+x>'),
         ('<Cmd+x>', '<Meta+x>'),
@@ -421,13 +421,10 @@ class TestKeySequence:
         ('', Qt.Key_Colon, Qt.AltModifier | Qt.ShiftModifier, ':',
          '<Alt+Shift+:>'),
 
-        # Swapping Control/Meta on macOS
-        ('', Qt.Key_A, Qt.ControlModifier, '',
-         '<Meta+A>' if utils.is_mac else '<Ctrl+A>'),
-        ('', Qt.Key_A, Qt.ControlModifier | Qt.ShiftModifier, '',
-         '<Meta+Shift+A>' if utils.is_mac else '<Ctrl+Shift+A>'),
-        ('', Qt.Key_A, Qt.MetaModifier, '',
-         '<Ctrl+A>' if utils.is_mac else '<Meta+A>'),
+        # Modifiers
+        ('', Qt.Key_A, Qt.ControlModifier, '', '<Ctrl+A>'),
+        ('', Qt.Key_A, Qt.ControlModifier | Qt.ShiftModifier, '', '<Ctrl+Shift+A>'),
+        ('', Qt.Key_A, Qt.MetaModifier, '', '<Meta+A>'),
 
         # Handling of Backtab
         ('', Qt.Key_Backtab, Qt.NoModifier, '', '<Backtab>'),
@@ -444,27 +441,6 @@ class TestKeySequence:
         new = seq.append_event(event)
         assert new == keyutils.KeySequence.parse(expected)
 
-    @pytest.mark.fake_os('mac')
-    @pytest.mark.parametrize('modifiers, expected', [
-        (Qt.ControlModifier,
-         Qt.MetaModifier),
-        (Qt.MetaModifier,
-         Qt.ControlModifier),
-        (Qt.ControlModifier | Qt.MetaModifier,
-         Qt.ControlModifier | Qt.MetaModifier),
-        (Qt.ControlModifier | Qt.ShiftModifier,
-         Qt.MetaModifier | Qt.ShiftModifier),
-        (Qt.MetaModifier | Qt.ShiftModifier,
-         Qt.ControlModifier | Qt.ShiftModifier),
-        (Qt.ShiftModifier, Qt.ShiftModifier),
-    ])
-    def test_fake_mac(self, modifiers, expected):
-        """Make sure Control/Meta are swapped with a simulated Mac."""
-        seq = keyutils.KeySequence()
-        info = keyutils.KeyInfo(key=Qt.Key_A, modifiers=modifiers)
-        new = seq.append_event(info.to_event())
-        assert new[0] == keyutils.KeyInfo(Qt.Key_A, expected)
-
     @pytest.mark.parametrize('key', [Qt.Key_unknown, 0x0])
     def test_append_event_invalid(self, key):
         seq = keyutils.KeySequence()
