@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2015-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2015-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -15,7 +15,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <http://www.gnu.org/licenses/>.
+# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
 
 """Tests for qutebrowser.misc.guiprocess."""
 
@@ -25,7 +25,7 @@ import pytest
 from PyQt5.QtCore import QProcess
 
 from qutebrowser.misc import guiprocess
-from qutebrowser.utils import usertypes
+from qutebrowser.utils import usertypes, utils
 from qutebrowser.browser import qutescheme
 
 
@@ -54,8 +54,8 @@ def fake_proc(monkeypatch, stubs):
 
 def test_start(proc, qtbot, message_mock, py_proc):
     """Test simply starting a process."""
-    with qtbot.waitSignal(proc.started, timeout=10000), \
-         qtbot.waitSignal(proc.finished, timeout=10000):
+    with qtbot.waitSignals([proc.started, proc.finished], timeout=10000,
+                           order='strict'):
         argv = py_proc("import sys; print('test'); sys.exit(0)")
         proc.start(*argv)
 
@@ -70,8 +70,8 @@ def test_start_verbose(proc, qtbot, message_mock, py_proc):
     """Test starting a process verbosely."""
     proc.verbose = True
 
-    with qtbot.waitSignal(proc.started, timeout=10000), \
-         qtbot.waitSignal(proc.finished, timeout=10000):
+    with qtbot.waitSignals([proc.started, proc.finished], timeout=10000,
+                           order='strict'):
         argv = py_proc("import sys; print('test'); sys.exit(0)")
         proc.start(*argv)
 
@@ -99,8 +99,9 @@ def test_start_output_message(proc, qtbot, caplog, message_mock, py_proc,
     code.append("sys.exit(0)")
 
     with caplog.at_level(logging.ERROR, 'message'):
-        with qtbot.waitSignal(proc.started, timeout=10000), \
-             qtbot.waitSignal(proc.finished, timeout=10000):
+        with qtbot.waitSignals([proc.started, proc.finished],
+                               timeout=10000,
+                               order='strict'):
             argv = py_proc(';'.join(code))
             proc.start(*argv)
 
@@ -146,8 +147,8 @@ def test_start_env(monkeypatch, qtbot, py_proc):
         sys.exit(0)
     """)
 
-    with qtbot.waitSignal(proc.started, timeout=10000), \
-         qtbot.waitSignal(proc.finished, timeout=10000):
+    with qtbot.waitSignals([proc.started, proc.finished], timeout=10000,
+                           order='strict'):
         proc.start(*argv)
 
     data = qutescheme.spawn_output
@@ -186,12 +187,12 @@ def test_double_start(qtbot, proc, py_proc):
 
 def test_double_start_finished(qtbot, proc, py_proc):
     """Test starting a GUIProcess twice (with the first call finished)."""
-    with qtbot.waitSignal(proc.started, timeout=10000), \
-         qtbot.waitSignal(proc.finished, timeout=10000):
+    with qtbot.waitSignals([proc.started, proc.finished], timeout=10000,
+                           order='strict'):
         argv = py_proc("import sys; sys.exit(0)")
         proc.start(*argv)
-    with qtbot.waitSignal(proc.started, timeout=10000), \
-         qtbot.waitSignal(proc.finished, timeout=10000):
+    with qtbot.waitSignals([proc.started, proc.finished], timeout=10000,
+                           order='strict'):
         argv = py_proc("import sys; sys.exit(0)")
         proc.start(*argv)
 
@@ -236,6 +237,22 @@ def test_exit_unsuccessful(qtbot, proc, message_mock, py_proc, caplog):
     assert msg.text == expected
 
 
+def test_exit_crash(qtbot, proc, message_mock, py_proc, caplog):
+    with caplog.at_level(logging.ERROR):
+        with qtbot.waitSignal(proc.finished, timeout=10000):
+            proc.start(*py_proc("""
+                import os, signal
+                os.kill(os.getpid(), signal.SIGSEGV)
+            """))
+
+    expected = (
+        "Testprocess exited with status 11, see :messages for details."
+        if utils.is_windows else "Testprocess crashed."
+    )
+    msg = message_mock.getmsg(usertypes.MessageLevel.error)
+    assert msg.text == expected
+
+
 @pytest.mark.parametrize('stream', ['stdout', 'stderr'])
 def test_exit_unsuccessful_output(qtbot, proc, caplog, py_proc, stream):
     """When a process fails, its output should be logged."""
@@ -266,8 +283,8 @@ def test_exit_successful_output(qtbot, proc, py_proc, stream):
 
 def test_stdout_not_decodable(proc, qtbot, message_mock, py_proc):
     """Test handling malformed utf-8 in stdout."""
-    with qtbot.waitSignal(proc.started, timeout=10000), \
-         qtbot.waitSignal(proc.finished, timeout=10000):
+    with qtbot.waitSignals([proc.started, proc.finished], timeout=10000,
+                           order='strict'):
         argv = py_proc(r"""
             import sys
             # Using \x81 because it's invalid in UTF-8 and CP1252
