@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -15,7 +15,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <http://www.gnu.org/licenses/>.
+# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
 
 
 """Tests for qutebrowser.utils.qtutils."""
@@ -23,6 +23,7 @@
 import io
 import os
 import os.path
+import dataclasses
 import unittest
 import unittest.mock
 
@@ -53,23 +54,25 @@ else:
 @pytest.mark.parametrize(['qversion', 'compiled', 'pyqt', 'version', 'exact',
                           'expected'], [
     # equal versions
-    ('5.4.0', None, None, '5.4.0', False, True),
-    ('5.4.0', None, None, '5.4.0', True, True),  # exact=True
-    ('5.4.0', None, None, '5.4', True, True),  # without trailing 0
+    ('5.14.0', None, None, '5.14.0', False, True),
+    ('5.14.0', None, None, '5.14.0', True, True),  # exact=True
+    ('5.14.0', None, None, '5.14', True, True),  # without trailing 0
     # newer version installed
-    ('5.4.1', None, None, '5.4', False, True),
-    ('5.4.1', None, None, '5.4', True, False),  # exact=True
+    ('5.14.1', None, None, '5.14', False, True),
+    ('5.14.1', None, None, '5.14', True, False),  # exact=True
     # older version installed
-    ('5.3.2', None, None, '5.4', False, False),
-    ('5.3.0', None, None, '5.3.2', False, False),
-    ('5.3.0', None, None, '5.3.2', True, False),  # exact=True
+    ('5.13.2', None, None, '5.14', False, False),
+    ('5.13.0', None, None, '5.13.2', False, False),
+    ('5.13.0', None, None, '5.13.2', True, False),  # exact=True
     # compiled=True
     # new Qt runtime, but compiled against older version
-    ('5.4.0', '5.3.0', '5.4.0', '5.4.0', False, False),
+    ('5.14.0', '5.13.0', '5.14.0', '5.14.0', False, False),
     # new Qt runtime, compiled against new version, but old PyQt
-    ('5.4.0', '5.4.0', '5.3.0', '5.4.0', False, False),
+    ('5.14.0', '5.14.0', '5.13.0', '5.14.0', False, False),
     # all up-to-date
-    ('5.4.0', '5.4.0', '5.4.0', '5.4.0', False, True),
+    ('5.14.0', '5.14.0', '5.14.0', '5.14.0', False, True),
+    # dev suffix
+    ('5.15.1', '5.15.1', '5.15.2.dev2009281246', '5.15.0', False, True),
 ])
 # pylint: enable=bad-continuation
 def test_version_check(monkeypatch, qversion, compiled, pyqt, version, exact,
@@ -119,7 +122,7 @@ def test_is_new_qtwebkit(monkeypatch, version, is_new):
 ])
 def test_is_single_process(monkeypatch, stubs, backend, arguments, single_process):
     qapp = stubs.FakeQApplication(arguments=arguments)
-    monkeypatch.setattr(qtutils, 'QApplication', qapp)
+    monkeypatch.setattr(qtutils.objects, 'qapp', qapp)
     monkeypatch.setattr(qtutils.objects, 'backend', backend)
     assert qtutils.is_single_process() == single_process
 
@@ -917,14 +920,14 @@ class TestEventLoop:
     def _double_exec(self):
         """Slot which gets called from timers to assert double-exec fails."""
         with pytest.raises(AssertionError):
-            self.loop.exec_()
+            self.loop.exec()
 
     def test_normal_exec(self):
         """Test exec_ without double-executing."""
         self.loop = qtutils.EventLoop()
         QTimer.singleShot(100, self._assert_executing)
         QTimer.singleShot(200, self.loop.quit)
-        self.loop.exec_()
+        self.loop.exec()
         assert not self.loop._executing
 
     def test_double_exec(self):
@@ -934,5 +937,109 @@ class TestEventLoop:
         QTimer.singleShot(200, self._double_exec)
         QTimer.singleShot(300, self._assert_executing)
         QTimer.singleShot(400, self.loop.quit)
-        self.loop.exec_()
+        self.loop.exec()
         assert not self.loop._executing
+
+
+class Color(QColor):
+
+    """A QColor with a nicer repr()."""
+
+    def __repr__(self):
+        return utils.get_repr(self, constructor=True, red=self.red(),
+                              green=self.green(), blue=self.blue(),
+                              alpha=self.alpha())
+
+
+class TestInterpolateColor:
+
+    @dataclasses.dataclass
+    class Colors:
+
+        white: Color
+        black: Color
+
+    @pytest.fixture
+    def colors(self):
+        """Example colors to be used."""
+        return self.Colors(Color('white'), Color('black'))
+
+    def test_invalid_start(self, colors):
+        """Test an invalid start color."""
+        with pytest.raises(qtutils.QtValueError):
+            qtutils.interpolate_color(Color(), colors.white, 0)
+
+    def test_invalid_end(self, colors):
+        """Test an invalid end color."""
+        with pytest.raises(qtutils.QtValueError):
+            qtutils.interpolate_color(colors.white, Color(), 0)
+
+    @pytest.mark.parametrize('perc', [-1, 101])
+    def test_invalid_percentage(self, colors, perc):
+        """Test an invalid percentage."""
+        with pytest.raises(ValueError):
+            qtutils.interpolate_color(colors.white, colors.white, perc)
+
+    def test_invalid_colorspace(self, colors):
+        """Test an invalid colorspace."""
+        with pytest.raises(ValueError):
+            qtutils.interpolate_color(colors.white, colors.black, 10, QColor.Cmyk)
+
+    @pytest.mark.parametrize('colorspace', [QColor.Rgb, QColor.Hsv,
+                                            QColor.Hsl])
+    def test_0_100(self, colors, colorspace):
+        """Test 0% and 100% in different colorspaces."""
+        white = qtutils.interpolate_color(colors.white, colors.black, 0, colorspace)
+        black = qtutils.interpolate_color(colors.white, colors.black, 100, colorspace)
+        assert Color(white) == colors.white
+        assert Color(black) == colors.black
+
+    def test_interpolation_rgb(self):
+        """Test an interpolation in the RGB colorspace."""
+        color = qtutils.interpolate_color(
+            Color(0, 40, 100), Color(0, 20, 200), 50, QColor.Rgb)
+        assert Color(color) == Color(0, 30, 150)
+
+    def test_interpolation_hsv(self):
+        """Test an interpolation in the HSV colorspace."""
+        start = Color()
+        stop = Color()
+        start.setHsv(0, 40, 100)
+        stop.setHsv(0, 20, 200)
+        color = qtutils.interpolate_color(start, stop, 50, QColor.Hsv)
+        expected = Color()
+        expected.setHsv(0, 30, 150)
+        assert Color(color) == expected
+
+    def test_interpolation_hsl(self):
+        """Test an interpolation in the HSL colorspace."""
+        start = Color()
+        stop = Color()
+        start.setHsl(0, 40, 100)
+        stop.setHsl(0, 20, 200)
+        color = qtutils.interpolate_color(start, stop, 50, QColor.Hsl)
+        expected = Color()
+        expected.setHsl(0, 30, 150)
+        assert Color(color) == expected
+
+    @pytest.mark.parametrize('colorspace', [QColor.Rgb, QColor.Hsv,
+                                            QColor.Hsl])
+    def test_interpolation_alpha(self, colorspace):
+        """Test interpolation of colorspace's alpha."""
+        start = Color(0, 0, 0, 30)
+        stop = Color(0, 0, 0, 100)
+        color = qtutils.interpolate_color(start, stop, 50, colorspace)
+        expected = Color(0, 0, 0, 65)
+        assert Color(color) == expected
+
+    @pytest.mark.parametrize('percentage, expected', [
+        (0, (0, 0, 0)),
+        (99, (0, 0, 0)),
+        (100, (255, 255, 255)),
+    ])
+    def test_interpolation_none(self, percentage, expected):
+        """Test an interpolation with a gradient turned off."""
+        color = qtutils.interpolate_color(
+            Color(0, 0, 0), Color(255, 255, 255), percentage, None)
+        assert isinstance(color, QColor)
+        assert Color(color) == Color(*expected)
