@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -15,7 +15,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <http://www.gnu.org/licenses/>.
+# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
 
 """Tests for qutebrowser.utils.standarddir."""
 
@@ -28,7 +28,6 @@ import textwrap
 import logging
 import subprocess
 
-import attr
 from PyQt5.QtCore import QStandardPaths
 import pytest
 
@@ -108,7 +107,7 @@ def test_fake_windows(tmpdir, monkeypatch, what):
 def test_fake_haiku(tmpdir, monkeypatch):
     """Test getting data dir on HaikuOS."""
     locations = {
-        QStandardPaths.DataLocation: '',
+        QStandardPaths.AppDataLocation: '',
         QStandardPaths.ConfigLocation: str(tmpdir / 'config' / APPNAME),
     }
     monkeypatch.setattr(standarddir.QStandardPaths, 'writableLocation',
@@ -129,14 +128,14 @@ class TestWritableLocation:
             'qutebrowser.utils.standarddir.QStandardPaths.writableLocation',
             lambda typ: '')
         with pytest.raises(standarddir.EmptyValueError):
-            standarddir._writable_location(QStandardPaths.DataLocation)
+            standarddir._writable_location(QStandardPaths.AppDataLocation)
 
     def test_sep(self, monkeypatch):
         """Make sure the right kind of separator is used."""
         monkeypatch.setattr(standarddir.os, 'sep', '\\')
         monkeypatch.setattr(standarddir.os.path, 'join',
                             lambda *parts: '\\'.join(parts))
-        loc = standarddir._writable_location(QStandardPaths.DataLocation)
+        loc = standarddir._writable_location(QStandardPaths.AppDataLocation)
         assert '/' not in loc
         assert '\\' in loc
 
@@ -305,7 +304,7 @@ class TestInitCacheDirTag:
             Signature: 8a477f597d28d172789f06886806bc55
             # This file is a cache directory tag created by qutebrowser.
             # For information about cache directory tags, see:
-            #  http://www.brynosaurus.com/cachedir/
+            #  https://bford.info/cachedir/
         """).lstrip()
 
     def test_open_oserror(self, caplog, unwritable_tmp_path, monkeypatch):
@@ -392,133 +391,13 @@ class TestSystemData:
                                           fake_args):
         """Test that system-wide path is not used on non-Linux OS."""
         fake_args.basedir = str(tmpdir)
-        monkeypatch.setattr('sys.platform', "potato")
+        monkeypatch.setattr(sys, 'platform', 'potato')
         standarddir._init_data(args=fake_args)
         assert standarddir.data(system=True) == standarddir.data()
 
 
-class TestMoveWindowsAndMacOS:
-
-    """Test other invocations of _move_data."""
-
-    @pytest.fixture(autouse=True)
-    def patch_standardpaths(self, files, monkeypatch):
-        locations = {
-            QStandardPaths.DataLocation: str(files.local_data_dir),
-            QStandardPaths.AppDataLocation: str(files.roaming_data_dir),
-        }
-        monkeypatch.setattr(standarddir.QStandardPaths, 'writableLocation',
-                            locations.get)
-        monkeypatch.setattr(
-            standarddir, 'config', lambda auto=False:
-            str(files.auto_config_dir if auto else files.config_dir))
-
-    @pytest.fixture
-    def files(self, tmpdir):
-
-        @attr.s
-        class Files:
-
-            auto_config_dir = attr.ib()
-            config_dir = attr.ib()
-            local_data_dir = attr.ib()
-            roaming_data_dir = attr.ib()
-
-        return Files(
-            auto_config_dir=tmpdir / 'auto_config' / APPNAME,
-            config_dir=tmpdir / 'config' / APPNAME,
-            local_data_dir=tmpdir / 'data' / APPNAME,
-            roaming_data_dir=tmpdir / 'roaming-data' / APPNAME,
-        )
-
-    def test_move_macos(self, files):
-        """Test moving configs on macOS."""
-        (files.auto_config_dir / 'autoconfig.yml').ensure()
-        (files.auto_config_dir / 'quickmarks').ensure()
-        files.config_dir.ensure(dir=True)
-
-        standarddir._move_macos()
-
-        assert (files.auto_config_dir / 'autoconfig.yml').exists()
-        assert not (files.config_dir / 'autoconfig.yml').exists()
-        assert not (files.auto_config_dir / 'quickmarks').exists()
-        assert (files.config_dir / 'quickmarks').exists()
-
-    def test_move_windows(self, files):
-        """Test moving configs on Windows."""
-        (files.local_data_dir / 'data' / 'blocked-hosts').ensure()
-        (files.local_data_dir / 'qutebrowser.conf').ensure()
-        (files.local_data_dir / 'cache' / 'cachefile').ensure()
-
-        standarddir._move_windows()
-
-        assert (files.roaming_data_dir / 'data' / 'blocked-hosts').exists()
-        assert (files.roaming_data_dir / 'config' /
-                'qutebrowser.conf').exists()
-        assert not (files.roaming_data_dir / 'cache').exists()
-        assert (files.local_data_dir / 'cache' / 'cachefile').exists()
-
-
-class TestMove:
-
-    @pytest.fixture
-    def dirs(self, tmpdir):
-        @attr.s
-        class Dirs:
-
-            old = attr.ib()
-            new = attr.ib()
-            old_file = attr.ib()
-            new_file = attr.ib()
-
-        old_dir = tmpdir / 'old'
-        new_dir = tmpdir / 'new'
-        return Dirs(old=old_dir, new=new_dir,
-                    old_file=old_dir / 'file', new_file=new_dir / 'file')
-
-    def test_no_old_dir(self, dirs, caplog):
-        """Nothing should happen without any old directory."""
-        standarddir._move_data(str(dirs.old), str(dirs.new))
-        assert not any(message.startswith('Migrating data from')
-                       for message in caplog.messages)
-
-    @pytest.mark.parametrize('empty_dest', [True, False])
-    def test_moving_data(self, dirs, empty_dest):
-        dirs.old_file.ensure()
-        if empty_dest:
-            dirs.new.ensure(dir=True)
-
-        standarddir._move_data(str(dirs.old), str(dirs.new))
-        assert not dirs.old_file.exists()
-        assert dirs.new_file.exists()
-
-    def test_already_existing(self, dirs, caplog):
-        dirs.old_file.ensure()
-        dirs.new_file.ensure()
-
-        with caplog.at_level(logging.ERROR):
-            standarddir._move_data(str(dirs.old), str(dirs.new))
-
-        expected = "Failed to move data from {} as {} is non-empty!".format(
-            dirs.old, dirs.new)
-        assert caplog.messages[-1] == expected
-
-    def test_deleting_error(self, dirs, monkeypatch, mocker, caplog):
-        """When there was an error it should be logged."""
-        mock = mocker.Mock(side_effect=OSError('error'))
-        monkeypatch.setattr(standarddir.shutil, 'move', mock)
-        dirs.old_file.ensure()
-
-        with caplog.at_level(logging.ERROR):
-            standarddir._move_data(str(dirs.old), str(dirs.new))
-
-        expected = "Failed to move data from {} to {}: error".format(
-            dirs.old, dirs.new)
-        assert caplog.messages[-1] == expected
-
-
 @pytest.mark.parametrize('args_kind', ['basedir', 'normal', 'none'])
-def test_init(mocker, tmpdir, monkeypatch, args_kind):
+def test_init(tmpdir, monkeypatch, args_kind):
     """Do some sanity checks for standarddir.init().
 
     Things like _init_cachedir_tag() are tested in more detail in other tests.
@@ -527,8 +406,6 @@ def test_init(mocker, tmpdir, monkeypatch, args_kind):
 
     monkeypatch.setenv('HOME', str(tmpdir))
 
-    m_windows = mocker.patch('qutebrowser.utils.standarddir._move_windows')
-    m_mac = mocker.patch('qutebrowser.utils.standarddir._move_macos')
     if args_kind == 'normal':
         args = types.SimpleNamespace(basedir=None)
     elif args_kind == 'basedir':
@@ -540,19 +417,6 @@ def test_init(mocker, tmpdir, monkeypatch, args_kind):
     standarddir.init(args)
 
     assert standarddir._locations != {}
-    if args_kind == 'normal':
-        if utils.is_mac:
-            m_windows.assert_not_called()
-            assert m_mac.called
-        elif utils.is_windows:
-            assert m_windows.called
-            m_mac.assert_not_called()
-        else:
-            m_windows.assert_not_called()
-            m_mac.assert_not_called()
-    else:
-        m_windows.assert_not_called()
-        m_mac.assert_not_called()
 
 
 @pytest.mark.linux
