@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2015-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2015-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -15,7 +15,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <http://www.gnu.org/licenses/>.
+# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
 
 """Tests for qutebrowser.utils.version."""
 
@@ -25,15 +25,14 @@ import collections
 import os.path
 import subprocess
 import contextlib
-import builtins  # noqa https://github.com/JBKahn/flake8-debugger/issues/20
+import builtins
 import types
 import importlib
 import logging
 import textwrap
 import datetime
+import dataclasses
 
-import attr
-import pkg_resources
 import pytest
 import hypothesis
 import hypothesis.strategies
@@ -77,7 +76,7 @@ from qutebrowser.browser import pdfjs
      """,
      version.DistributionInfo(
          id='ubuntu', parsed=version.Distribution.ubuntu,
-         version=pkg_resources.parse_version('14.4'),
+         version=utils.parse_version('14.4'),
          pretty='Ubuntu 14.04.5 LTS')),
     # Ubuntu 17.04
     ("""
@@ -90,7 +89,7 @@ from qutebrowser.browser import pdfjs
      """,
      version.DistributionInfo(
          id='ubuntu', parsed=version.Distribution.ubuntu,
-         version=pkg_resources.parse_version('17.4'),
+         version=utils.parse_version('17.4'),
          pretty='Ubuntu 17.04')),
     # Debian Jessie
     ("""
@@ -102,7 +101,7 @@ from qutebrowser.browser import pdfjs
      """,
      version.DistributionInfo(
          id='debian', parsed=version.Distribution.debian,
-         version=pkg_resources.parse_version('8'),
+         version=utils.parse_version('8'),
          pretty='Debian GNU/Linux 8 (jessie)')),
     # Void Linux
     ("""
@@ -133,7 +132,7 @@ from qutebrowser.browser import pdfjs
      """,
      version.DistributionInfo(
          id='fedora', parsed=version.Distribution.fedora,
-         version=pkg_resources.parse_version('25'),
+         version=utils.parse_version('25'),
          pretty='Fedora 25 (Twenty Five)')),
     # OpenSUSE
     ("""
@@ -146,7 +145,7 @@ from qutebrowser.browser import pdfjs
      """,
      version.DistributionInfo(
          id='opensuse', parsed=version.Distribution.opensuse,
-         version=pkg_resources.parse_version('42.2'),
+         version=utils.parse_version('42.2'),
          pretty='openSUSE Leap 42.2')),
     # Linux Mint
     ("""
@@ -159,7 +158,7 @@ from qutebrowser.browser import pdfjs
      """,
      version.DistributionInfo(
          id='linuxmint', parsed=version.Distribution.linuxmint,
-         version=pkg_resources.parse_version('18.1'),
+         version=utils.parse_version('18.1'),
          pretty='Linux Mint 18.1')),
     # Manjaro
     ("""
@@ -188,7 +187,7 @@ from qutebrowser.browser import pdfjs
     """,
      version.DistributionInfo(
          id='org.kde.Platform', parsed=version.Distribution.kde_flatpak,
-         version=pkg_resources.parse_version('5.12'),
+         version=utils.parse_version('5.12'),
          pretty='KDE')),
     # No PRETTY_NAME
     ("""
@@ -221,7 +220,7 @@ def test_distribution(tmpdir, monkeypatch, os_release, expected):
     (None, False),
     (version.DistributionInfo(
         id='org.kde.Platform', parsed=version.Distribution.kde_flatpak,
-        version=pkg_resources.parse_version('5.12'),
+        version=utils.parse_version('5.12'),
         pretty='Unknown'), True),
     (version.DistributionInfo(
         id='arch', parsed=version.Distribution.arch, version=None,
@@ -382,7 +381,7 @@ class TestGitStrSubprocess:
             if utils.is_windows:
                 # If we don't call this with shell=True it might fail under
                 # some environments on Windows...
-                # http://bugs.python.org/issue24493
+                # https://bugs.python.org/issue24493
                 subprocess.run(
                     'git -C "{}" {}'.format(tmpdir, ' '.join(args)),
                     env=env, check=True, shell=True)
@@ -545,7 +544,7 @@ class ImportFake:
 
     Attributes:
         modules: A dict mapping module names to bools. If True, the import will
-                 success. Otherwise, it'll fail with ImportError.
+                 succeed. Otherwise, it'll fail with ImportError.
         version_attribute: The name to use in the fake modules for the version
                            attribute.
         version: The version to use for the modules.
@@ -554,23 +553,8 @@ class ImportFake:
     """
 
     def __init__(self):
-        self.modules = collections.OrderedDict([
-            ('sip', True),
-            ('colorama', True),
-            ('pypeg2', True),
-            ('jinja2', True),
-            ('pygments', True),
-            ('yaml', True),
-            ('cssutils', True),
-            ('attr', True),
-            ('PyQt5.QtWebEngineWidgets', True),
-            ('PyQt5.QtWebEngine', True),
-            ('PyQt5.QtWebKitWidgets', True),
-        ])
-        self.no_version_attribute = ['sip',
-                                     'PyQt5.QtWebEngineWidgets',
-                                     'PyQt5.QtWebKitWidgets',
-                                     'PyQt5.QtWebEngine']
+        self.modules = collections.OrderedDict(
+            [(mod, True) for mod in version.MODULE_INFO])
         self.version_attribute = '__version__'
         self.version = '1.2.3'
         self._real_import = builtins.__import__
@@ -615,7 +599,7 @@ class ImportFake:
 def import_fake(monkeypatch):
     """Fixture to patch imports using ImportFake."""
     fake = ImportFake()
-    monkeypatch.setattr('builtins.__import__', fake.fake_import)
+    monkeypatch.setattr(builtins, '__import__', fake.fake_import)
     monkeypatch.setattr(version.importlib, 'import_module',
                         fake.fake_importlib_import)
     return fake
@@ -623,13 +607,14 @@ def import_fake(monkeypatch):
 
 class TestModuleVersions:
 
-    """Tests for _module_versions()."""
+    """Tests for _module_versions() and ModuleInfo."""
 
     def test_all_present(self, import_fake):
         """Test with all modules present in version 1.2.3."""
         expected = []
         for name in import_fake.modules:
-            if name in import_fake.no_version_attribute:
+            version.MODULE_INFO[name]._reset_cache()
+            if '__version__' not in version.MODULE_INFO[name]._version_attributes:
                 expected.append('{}: yes'.format(name))
             else:
                 expected.append('{}: 1.2.3'.format(name))
@@ -637,7 +622,7 @@ class TestModuleVersions:
 
     @pytest.mark.parametrize('module, idx, expected', [
         ('colorama', 1, 'colorama: no'),
-        ('cssutils', 6, 'cssutils: no'),
+        ('adblock', 5, 'adblock: no'),
     ])
     def test_missing_module(self, module, idx, expected, import_fake):
         """Test with a module missing.
@@ -648,8 +633,45 @@ class TestModuleVersions:
             expected: The expected text.
         """
         import_fake.modules[module] = False
+        # Needed after mocking the module
+        mod_info = version.MODULE_INFO[module]
+        mod_info._reset_cache()
+
         assert version._module_versions()[idx] == expected
 
+        for method_name, expected_result in [
+            ("is_installed", False),
+            ("is_usable", False),
+            ("get_version", None),
+            ("is_outdated", None)
+        ]:
+            method = getattr(mod_info, method_name)
+            # With hot cache
+            mod_info._initialize_info()
+            assert method() == expected_result
+            # With cold cache
+            mod_info._reset_cache()
+            assert method() == expected_result
+
+    def test_outdated_adblock(self, import_fake):
+        """Test that warning is shown when adblock module is outdated."""
+        mod_info = version.MODULE_INFO["adblock"]
+        fake_version = "0.1.0"
+
+        # Needed after mocking version attribute
+        mod_info._reset_cache()
+
+        assert mod_info.min_version is not None
+        assert fake_version < mod_info.min_version
+        import_fake.version = fake_version
+
+        assert mod_info.is_installed()
+        assert mod_info.is_outdated()
+        assert not mod_info.is_usable()
+
+        expected = f"adblock: {fake_version} (< {mod_info.min_version}, outdated)"
+        assert version._module_versions()[5] == expected
+
     @pytest.mark.parametrize('attribute, expected_modules', [
         ('VERSION', ['colorama']),
         ('SIP_VERSION_STR', ['sip']),
@@ -666,23 +688,33 @@ class TestModuleVersions:
             expected: The expected return value.
         """
         import_fake.version_attribute = attribute
+
+        for mod_info in version.MODULE_INFO.values():
+            # Invalidate the "version cache" since we just mocked some of the
+            # attributes.
+            mod_info._reset_cache()
+
         expected = []
         for name in import_fake.modules:
+            mod_info = version.MODULE_INFO[name]
             if name in expected_modules:
+                assert mod_info.get_version() == "1.2.3"
                 expected.append('{}: 1.2.3'.format(name))
             else:
+                assert mod_info.get_version() is None
                 expected.append('{}: yes'.format(name))
+
         assert version._module_versions() == expected
 
     @pytest.mark.parametrize('name, has_version', [
         ('sip', False),
         ('colorama', True),
-        ('pypeg2', True),
         ('jinja2', True),
         ('pygments', True),
         ('yaml', True),
-        ('cssutils', True),
-        ('attr', True),
+        ('adblock', True),
+        ('dataclasses', False),
+        ('importlib_resources', False),
     ])
     def test_existing_attributes(self, name, has_version):
         """Check if all dependencies have an expected __version__ attribute.
@@ -694,9 +726,7 @@ class TestModuleVersions:
             name: The name of the module to check.
             has_version: Whether a __version__ attribute is expected.
         """
-        if name == 'cssutils':
-            pytest.importorskip(name)
-        module = importlib.import_module(name)
+        module = pytest.importorskip(name)
         assert hasattr(module, '__version__') == has_version
 
     def test_existing_sip_attribute(self):
@@ -913,18 +943,18 @@ class TestChromiumVersion:
         assert version._chromium_version() == 'avoided'
 
 
-@attr.s
+@dataclasses.dataclass
 class VersionParams:
 
-    name = attr.ib()
-    git_commit = attr.ib(True)
-    frozen = attr.ib(False)
-    qapp = attr.ib(True)
-    with_webkit = attr.ib(True)
-    known_distribution = attr.ib(True)
-    ssl_support = attr.ib(True)
-    autoconfig_loaded = attr.ib(True)
-    config_py_loaded = attr.ib(True)
+    name: str
+    git_commit: bool = True
+    frozen: bool = False
+    qapp: bool = True
+    with_webkit: bool = True
+    known_distribution: bool = True
+    ssl_support: bool = True
+    autoconfig_loaded: bool = True
+    config_py_loaded: bool = True
 
 
 @pytest.mark.parametrize('params', [
@@ -959,10 +989,8 @@ def test_version_info(params, stubs, monkeypatch, config_stub):
         'platform.architecture': lambda: ('ARCHITECTURE', ''),
         '_os_info': lambda: ['OS INFO 1', 'OS INFO 2'],
         '_path_info': lambda: {'PATH DESC': 'PATH NAME'},
-        'QApplication': (stubs.FakeQApplication(style='STYLE',
-                                                platform_name='PLATFORM')
-                         if params.qapp else
-                         stubs.FakeQApplication(instance=None)),
+        'objects.qapp': (stubs.FakeQApplication(style='STYLE', platform_name='PLATFORM')
+                         if params.qapp else None),
         'QLibraryInfo.location': (lambda _loc: 'QT PATH'),
         'sql.version': lambda: 'SQLITE VERSION',
         '_uptime': lambda: datetime.timedelta(hours=1, minutes=23, seconds=45),
@@ -1023,7 +1051,7 @@ def test_version_info(params, stubs, monkeypatch, config_stub):
     substitutions['ssl'] = 'SSL VERSION' if params.ssl_support else 'no'
 
     for name, val in patches.items():
-        monkeypatch.setattr('qutebrowser.utils.version.' + name, val)
+        monkeypatch.setattr(f'qutebrowser.utils.version.{name}', val)
 
     if params.frozen:
         monkeypatch.setattr(sys, 'frozen', True, raising=False)
@@ -1138,9 +1166,8 @@ def pbclient(stubs):
 
 def test_pastebin_version(pbclient, message_mock, monkeypatch, qtbot):
     """Test version.pastebin_version() sets the url."""
-    monkeypatch.setattr('qutebrowser.utils.version.version_info',
-                        lambda: "dummy")
-    monkeypatch.setattr('qutebrowser.utils.utils.log_clipboard', True)
+    monkeypatch.setattr(version, 'version_info', lambda: 'dummy')
+    monkeypatch.setattr(utils, 'log_clipboard', True)
 
     version.pastebin_version(pbclient)
     pbclient.success.emit("https://www.example.com/\n")
@@ -1153,8 +1180,7 @@ def test_pastebin_version(pbclient, message_mock, monkeypatch, qtbot):
 
 def test_pastebin_version_twice(pbclient, monkeypatch):
     """Test whether calling pastebin_version twice sends no data."""
-    monkeypatch.setattr('qutebrowser.utils.version.version_info',
-                        lambda: "dummy")
+    monkeypatch.setattr(version, 'version_info', lambda: 'dummy')
 
     version.pastebin_version(pbclient)
     pbclient.success.emit("https://www.example.com/\n")
@@ -1171,8 +1197,7 @@ def test_pastebin_version_twice(pbclient, monkeypatch):
 
 def test_pastebin_version_error(pbclient, caplog, message_mock, monkeypatch):
     """Test version.pastebin_version() with errors."""
-    monkeypatch.setattr('qutebrowser.utils.version.version_info',
-                        lambda: "dummy")
+    monkeypatch.setattr(version, 'version_info', lambda: 'dummy')
 
     version.pastebin_url = None
     with caplog.at_level(logging.ERROR):
@@ -1187,12 +1212,14 @@ def test_pastebin_version_error(pbclient, caplog, message_mock, monkeypatch):
 
 def test_uptime(monkeypatch, qapp):
     """Test _uptime runs and check if microseconds are dropped."""
+    monkeypatch.setattr(objects, 'qapp', qapp)
+
     launch_time = datetime.datetime(1, 1, 1, 1, 1, 1, 1)
     monkeypatch.setattr(qapp, "launch_time", launch_time, raising=False)
 
     class FakeDateTime(datetime.datetime):
         now = lambda x=datetime.datetime(1, 1, 1, 1, 1, 1, 2): x
-    monkeypatch.setattr('datetime.datetime', FakeDateTime)
+    monkeypatch.setattr(datetime, 'datetime', FakeDateTime)
 
     uptime_delta = version._uptime()
     assert uptime_delta == datetime.timedelta(0)
